// $Id: ActionFrame.cpp,v 1.69 2014/02/25 09:53:42 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ActionFrame.h"
#include "ActionFrame.moc"
#include "BaseIconNames.h"
#include "Debug.h"
#include "File.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "WarningDialog.h"
#include "XmlOptions.h"

#include <QFile>
#include <QShortcut>
#include <QTextStream>

//_______________________________________________
ActionFrame::ActionFrame( QWidget* parent ):
    QGroupBox( parent ),
    Counter( "ActionFrame" ),
    autoBibtex_( false )
{

    Debug::Throw( "ActionFrame::ActrionFrame.\n" );
    setLayout( new QVBoxLayout() );
    layout()->setMargin( 5 );
    layout()->setSpacing( 2 );

    // create checkboxes
    box_ = new QWidget( this );
    layout()->addWidget( box_ );

    QVBoxLayout* checkboxLayout( new QVBoxLayout() );
    checkboxLayout->setSpacing( 2 );
    checkboxLayout->setMargin( 0 );
    box_->setLayout( checkboxLayout );

    // create checkboxes
    checkbox_.insert( LatexCommand::Latex, new QCheckBox( tr( "Latex" ), this ) );
    checkbox_.insert( LatexCommand::PDFLatex, new QCheckBox( tr( "PdfLatex" ), this ) );
    checkbox_.insert( LatexCommand::Bibtex, new QCheckBox( tr( "Bibtex" ), this ) );
    checkbox_.insert( LatexCommand::DviPS, new QCheckBox( tr( "Dvi to PS" ), this ) );
    checkbox_.insert( LatexCommand::DviPDF, new QCheckBox( tr( "Dvi to PDF" ), this ) );
    checkbox_.insert( LatexCommand::PS2PDF, new QCheckBox( tr( "PS to PDF" ), this ) );
    checkbox_.insert( LatexCommand::Check, new QCheckBox( tr( "Check" ), this ) );
    checkbox_.insert( LatexCommand::Clean, new QCheckBox( tr( "Clean" ), this ) );

    // apply state from options
    unsigned int actionState( XmlOptions::get().get<unsigned int>( "ACTIONS" ) );
    for( CheckBoxMap::const_iterator iter = checkbox_.begin(); iter != checkbox_.end(); ++iter )
    {
        checkboxLayout->addWidget( iter.value() );
        if( actionState & (1<<iter.key()) ) iter.value()->setChecked( true );
        connect( iter.value(), SIGNAL(toggled(bool)), SLOT(_updateButton(bool)) );
    }

    // run button
    button_ = new QPushButton( tr( "Run" ), this );
    button_->setIcon( IconEngine::get( IconNames::DialogAccept ) );
    layout()->addWidget( button_ );
    button_->setToolTip( tr( "Run/abort selected actions" ) );
    connect( button_, SIGNAL(clicked()), SLOT(_buttonPressed()) );
    state_ = StateIdle;

    // close accelerator
    new QShortcut( Qt::CTRL + Qt::Key_C, this, SLOT(_abort()) );

}

//_______________________________________________
void ActionFrame::setAutoBibtex( bool value )
{
    Debug::Throw( "ActionFrame::setAutoBibtex.\n" );
    autoBibtex_ = value;
    checkbox_[LatexCommand::Bibtex]->setVisible( !value );
}

//_______________________________________________
void ActionFrame::setBusy( void )
{

    Debug::Throw( "ActionFrame::setBusy.\n" );

    // change button text and status
    button_->setText( tr( "Abort" ) );
    button_->setIcon( IconEngine::get( IconNames::DialogCancel ) );

    // disable widgets
    box_->setEnabled( false );

    // save state
    state_ = StateRunning;

}

//_______________________________________________
void ActionFrame::setIdle( void )
{

    Debug::Throw( "ActionFrame::setIdle.\n" );

    // process warnings
    if( !warnings_.isEmpty() )
    {
        QString buffer;
        if( warnings_.size() == 1 ) buffer = QString( tr( "Warning: %1" ) ).arg( warnings_[0] );
        else buffer = QString( tr( "Warning:\n%1" ) ).arg( warnings_.join( "\n" ) );
        WarningDialog dialog( this, buffer );
        dialog.setDetails( warningLog_ );

        warnings_.clear();
        warningLog_.clear();
        dialog.exec();

    }

    // change button text and status
    button_->setText( tr( "Run" ) );
    button_->setIcon( IconEngine::get( IconNames::DialogAccept ) );

    // enable widgets
    box_->setEnabled( true );

    // save state
    state_ = StateIdle;

}

//_______________________________________________
void ActionFrame::updateBibtexFile( const File& value )
{ bibtexFile_ = value; }

//_______________________________________________
void ActionFrame::_updateButton( bool value )
{
    if( value ) button_->setEnabled( true );
    else {

        bool enabled( false );
        for( CheckBoxMap::const_iterator iter = checkbox_.begin(); iter != checkbox_.end() && !enabled; ++iter )
        {  if( iter.value()->isChecked() ) enabled = true; }

        button_->setEnabled( enabled );

    }


}

//_______________________________________________
void ActionFrame::_buttonPressed( void )
{

    if( state_ == StateRunning ) emit commandsAborted();
    else {

        // clear warnings
        warnings_.clear();
        warningLog_.clear();

        // list of latex commands
        LatexCommand::List commands;

        // add a clean call prior to running anything, if selected
        /* this can recover dirty states from previous failed attempts */
        if( _isChecked( LatexCommand::Clean ) ) commands << (LatexCommand( LatexCommand::Clean ) << _file() );

        // fill actions depending on CheckBoxes
        // latex
        if( _isChecked( LatexCommand::Latex ) )
        {

            emit recentFileAdded( _file().truncatedName() );
            commands << (LatexCommand( LatexCommand::Latex ) << _file() );

            emit cyclesAvailable( 1 );

        }

        // pdf latex
        if( _isChecked( LatexCommand::PDFLatex ) ) {

            emit recentFileAdded( _file().truncatedName() );
            commands << (LatexCommand( LatexCommand::PDFLatex ) << _file() );

            emit cyclesAvailable( 1 );

        }

        // bibtex
        const bool needBibtex( !bibtexFile_.isEmpty() && bibtexFile_.exists() );
        if( _isChecked( LatexCommand::Bibtex ) || ( autoBibtex_ && needBibtex ) )
        {
            // need to run bibtex 3 time to have references right
            commands << (LatexCommand( LatexCommand::Bibtex ) << _file() );
            commands << (LatexCommand( LatexCommand::Bibtex ) << _file() );
            commands << (LatexCommand( LatexCommand::Bibtex ) << _file() );

            if( _isChecked( LatexCommand::Latex ) )
            {

                commands << (LatexCommand( LatexCommand::Latex ) << _file() );
                commands << (LatexCommand( LatexCommand::Latex ) << _file() );
                emit cyclesAvailable( 3 );

            } else if( _isChecked( LatexCommand::PDFLatex ) ) {

                commands << (LatexCommand( LatexCommand::PDFLatex ) << _file() );
                commands << (LatexCommand( LatexCommand::PDFLatex ) << _file() );
                emit cyclesAvailable( 3 );

            }

        } else if( _isChecked( LatexCommand::Latex ) ) {

            // run latex a second time
            commands << (LatexCommand( LatexCommand::Latex ) << _file() );
            emit cyclesAvailable( 2 );

        } else if( _isChecked( LatexCommand::PDFLatex ) ) {

            // run latex a second time
            commands << (LatexCommand( LatexCommand::PDFLatex ) << _file() );
            emit cyclesAvailable( 2 );

        }

        // dvips
        if( _isChecked( LatexCommand::DviPS ) ) commands << (LatexCommand( LatexCommand::DviPS ) << _file() );

        // dvipdf
        if( _isChecked( LatexCommand::DviPDF ) ) commands << (LatexCommand( LatexCommand::DviPDF ) << _file() );

        // ps2pdf
        if( _isChecked( LatexCommand::PS2PDF ) ) commands << (LatexCommand( LatexCommand::PS2PDF ) << _file() );

        // check
        if( _isChecked( LatexCommand::Check ) )
        {
            commands << (LatexCommand( LatexCommand::CheckCitations ) << _file() );
            commands << (LatexCommand( LatexCommand::CheckCitationOrder ) << _file() );
            commands << (LatexCommand( LatexCommand::CheckLabels ) << _file() );
            commands << (LatexCommand( LatexCommand::CheckReferences ) << _file() );
        }

        // clean
        if( _isChecked( LatexCommand::Clean ) ) commands << (LatexCommand( LatexCommand::Clean ) << _file() );
        if( !commands.empty() )
        {
            _updateActionState();
            emit commandsRequested( commands );
        }

        else InformationDialog( this, tr( "No action taken." ) ).exec();

    }

    return;

}

//_______________________________________________
void ActionFrame::_abort( void )
{ if( state_ == StateRunning ) emit commandsAborted(); }

//_______________________________________________
void ActionFrame::processWarning( const QString& warning )
{ warnings_.append( warning ); }

//_______________________________________________
void ActionFrame::appendToLog( const QString& warning )
{ warningLog_ += warning; }

//_______________________________________________
void ActionFrame::_updateActionState( void )
{

    Debug::Throw( "ActionFrame::_updateActionState.\n" );

    // create checkboxes
    unsigned int actionState( 0 );
    for( CheckBoxMap::const_iterator iter = checkbox_.begin(); iter != checkbox_.end(); ++iter )
    { if( iter.value()->isChecked() ) actionState |= (1<<iter.key()); }

    XmlOptions::get().set<unsigned int>("ACTIONS", actionState );
    XmlOptions::write();
}
