// $Id: Application.cpp,v 1.45 2014/02/10 10:34:30 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Application.h"
#include "Application.moc"

#include "ActionFrame.h"
#include "ButtonFrame.h"
#include "Config.h"
#include "ConfigurationDialog.h"
#include "Debug.h"
#include "LatexInterface.h"
#include "Menu.h"
#include "MainWindow.h"
#include "QtUtil.h"
#include "QuestionDialog.h"
#include "RecentFilesMenu.h"
#include "XmlFileList.h"
#include "XmlOptions.h"

#include <QApplication>
#include <QMessageBox>
#include <QPixmap>
#include <QTextStream>

//____________________________________________
Application::Application( CommandLineArguments arguments ) :
    BaseApplication( 0, arguments  ),
    Counter( "Application" ),
    recentFiles_( 0 ),
    mainWindow_(0)
{}

//____________________________________________
Application::~Application( void )
{
    Debug::Throw( "Application::~Application.\n" );
    if( recentFiles_ ) delete recentFiles_;
}

//____________________________________________
bool Application::initApplicationManager( void )
{
    Debug::Throw( "Application::initApplicationManager.\n" );

    // retrieve files from arguments and expand if needed
    CommandLineParser parser( commandLineParser( _arguments() ) );
    QStringList& orphans( parser.orphans() );
    for( QStringList::iterator iter = orphans.begin(); iter != orphans.end(); ++iter )
    { if( !iter->isEmpty() ) (*iter) = File( *iter ).expand(); }

    // replace arguments
    _setArguments( parser.arguments() );

    // base class initialization
    return BaseApplication::initApplicationManager();

}

//____________________________________________
bool Application::realizeWidget( void )
{
    Debug::Throw( "Application::RealizeWidget.\n" );

    // check if the method has already been called.
    if( !BaseApplication::realizeWidget() ) return false;

    // file list
    recentFiles_ = new XmlFileList();
    recentFiles_->setCheck( false );

    // set icon Pixmap
    mainWindow_ = new MainWindow(0);

    // configuration
    emit configurationChanged();

    // connections
    connect( &mainWindow().interface(), SIGNAL(busy()), SLOT(busy()) );
    connect( &mainWindow().interface(), SIGNAL(ready()), SLOT(idle()) );

    // recent files addition
    connect( &mainWindow().actionFrame(), SIGNAL(recentFileAdded(File)),  &recentFiles(), SLOT(add(File)) );
    connect( &mainWindow().actionFrame(), SIGNAL(recentFileAdded(File)), &mainWindow().menu().recentFilesMenu(), SLOT(setCurrentFile(File)) );

    connect( &mainWindow().buttonFrame(), SIGNAL(recentFileAdded(File)),  &recentFiles(), SLOT(add(File)) );
    connect( &mainWindow().buttonFrame(), SIGNAL(recentFileAdded(File)), &mainWindow().menu().recentFilesMenu(), SLOT(setCurrentFile(File)) );

    // show main windows
    mainWindow().centerOnDesktop();
    mainWindow().show();

    const QStringList& filenames( commandLineParser( _arguments() ).orphans() );
    if( !filenames.isEmpty() ) mainWindow().open( FileRecord( File( filenames.front() ).expand() ) );
    else mainWindow().open( recentFiles().lastValidFile() );

    return true;

}

//____________________________________________
CommandLineParser Application::commandLineParser( CommandLineArguments arguments, bool ignoreWarnings ) const
{
    Debug::Throw( "Application::commandLineParser.\n" );
    CommandLineParser out( BaseApplication::commandLineParser() );
    out.setGroup( CommandLineParser::applicationGroupName );
    out.registerFlag( "--force", tr( "unconditionally open new file in existing window") );
    if( !arguments.isEmpty() ) out.parse( arguments, ignoreWarnings );
    return out;

}

//____________________________________________
void Application::usage( void ) const
{
    _usage( "textool", tr( "[options] [file]" ) );
    commandLineParser().usage();
    return;
}

//_________________________________________________
void Application::_configuration( void )
{

    Debug::Throw( "Application::configuration.\n" );
    emit saveConfiguration();

    ConfigurationDialog dialog;
    connect( &dialog, SIGNAL(configurationChanged()), SIGNAL(configurationChanged()) );
    dialog.centerOnWidget( qApp->activeWindow() );
    dialog.exec();

}

//________________________________________________
bool Application::_processCommand( Server::ServerCommand command )
{

    Debug::Throw( "Application::_processCommand.\n" );
    if( BaseApplication::_processCommand( command ) ) return true;
    if( command.command() == Server::ServerCommand::Raise )
    {
        if( mainWindow_ ) mainWindow().uniconify();

        const CommandLineParser parser( commandLineParser( command.arguments() ) );
        const QStringList filenames( parser.orphans() );
        if( filenames.isEmpty() ) return true;
        else if( parser.hasFlag( "--force" ) ) mainWindow().open( File( filenames.front() ) );
        else {
            const QString buffer = QString( tr( "Accept request for file '%1' ?" ) ).arg( filenames.front() );
            if( QuestionDialog( mainWindow_, buffer ).centerOnParent().exec() )
            { mainWindow().open( File( filenames.front() ) ); }
        }

        return true;

    } else return false;

}
