// $Id: Menu.cpp,v 1.54 2014/02/10 10:34:30 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Menu.h"
#include "Menu.moc"

#include "Application.h"
#include "Config.h"
#include "CustomPixmap.h"
#include "DebugMenu.h"
#include "DefaultHelpText.h"
#include "File.h"
#include "HelpManager.h"
#include "HelpText.h"
#include "MainWindow.h"
#include "RecentFilesMenu.h"
#include "Singleton.h"
#include "TexString.h"
#include "XmlOptions.h"

#include <QMenu>

//_______________________________________________
Menu::Menu( QWidget* parent ):
    QMenuBar( parent ),
    Counter( "Menu" ),
    thread_( this )
{
    Debug::Throw( "Menu::Menu.\n" );

    // generic menu
    QMenu *menu;

    Application &application( *Singleton::get().application<Application>() );
    MainWindow &mainWindow( *static_cast<MainWindow*>(window()) );

    // file menu
    menu = addMenu( tr( "File" ) );
    menu->addAction( &mainWindow.openAction() );

    // open previous menu
    recentFilesMenu_ = new RecentFilesMenu( this, application.recentFiles() );
    connect( recentFilesMenu_, SIGNAL(fileSelected(FileRecord)), &mainWindow, SLOT(open(FileRecord)) );
    menu->addMenu( recentFilesMenu_ );

    // exit button
    menu->addSeparator();
    menu->addAction( &application.closeAction() );

    // extension submenu
    editionMenu_ = addMenu( tr( "Edit" ) );
    connect( editionMenu_, SIGNAL(aboutToShow()), this, SLOT(_updateEditionMenu()) );

    // Tools menu
    menu = addMenu( tr( "Tools" ) );
    menu->addAction( tr( "Add latex Header" ), &mainWindow, SLOT(addLatexHeader()) );

    menu = menu->addMenu( tr( "French Accents" ) );
    menu->addAction( &mainWindow.toLatexAccentsAction() );
    menu->addAction( &mainWindow.toTextAccentsAction() );

    // Settings menu
    menu = addMenu( tr( "Settings" ) );
    menu->addAction( &application.configurationAction() );

    // help manager
    Base::HelpManager* help( new Base::HelpManager( this ) );
    help->setWindowTitle( tr( "Textool Handbook" ) );
    help->install( helpText );
    help->install( Base::helpText, false );

    // help menu
    menu = addMenu( tr( "Help" ) );
    menu->addAction( &help->displayAction() );
    menu->addSeparator();
    menu->addAction( &application.aboutAction() );
    menu->addAction( &application.aboutQtAction() );

    // debug menu
    menu->addSeparator();
    DebugMenu *debugMenu( new DebugMenu( menu ) );
    menu->addAction( debugMenu->menuAction() );
    debugMenu->menuAction()->setVisible( false );

    // thread connection
    connect( &thread_, SIGNAL(recordsAvailable(const FileRecord::List&,bool)), this, SLOT(_processRecords(const FileRecord::List&,bool)) );

}

//_______________________________________________
void Menu::_processRecords( const FileRecord::List& records, bool)
{

    // set file records validity
    for( ActionMap::iterator iter = editionActions_.begin(); iter != editionActions_.end(); ++iter )
    {
        FileRecord::List::const_iterator found = std::find_if(
            records.begin(),
            records.end(),
            FileRecord::SameFileFTor( iter->second.file() ) );
        if( found == records.end() ) continue;
        iter->first->setVisible( found->isValid() );
    }

    return;

}

//_________________________________________________
void Menu::_updateEditionMenu( void )
{
    Debug::Throw( "Menu::_updateEditionMenu.\n" );

    // clear actions
    editionMenu_->clear();
    editionActions_.clear();

    // retrieve current file
    MainWindow &mainWindow( *static_cast<MainWindow*>(window()) );
    File base( mainWindow.file() );

    // generic file
    File file;

    // generic action
    QAction* action;

    file = TexString( base ).addExtension( ".tex" );
    editionActions_ << ActionPair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) );
    connect( action, SIGNAL(triggered()), &mainWindow, SLOT(editTex()) );

    file = TexString( base ).addExtension( ".log" );
    editionActions_ << ActionPair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) );
    connect( action, SIGNAL(triggered()), &mainWindow, SLOT(editLog()) );

    file = TexString( base ).addExtension( ".aux" );
    editionActions_ << ActionPair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) );
    connect( action, SIGNAL(triggered()), &mainWindow, SLOT(editAux()) );

    file = TexString( base ).addExtension( ".toc" );
    editionActions_ << ActionPair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) );
    connect( action, SIGNAL(triggered()), &mainWindow, SLOT(editToc()) );

    file = TexString( base ).addExtension( ".tof" );
    editionActions_ << ActionPair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) );
    connect( action, SIGNAL(triggered()), &mainWindow, SLOT(editTof()) );

    // bibliography
    if( mainWindow.hasBibtexFile() )
    {
        editionActions_ << ActionPair( action = new QAction( mainWindow.bibtexFile().localName(), editionMenu_ ), mainWindow.bibtexFile() );
        connect( action, SIGNAL(triggered()), &mainWindow, SLOT(editBib()) );
    }

    file = TexString( base ).addExtension( ".ps" );
    editionActions_ << ActionPair( action = new QAction( file.localName(), editionMenu_ ), FileRecord( file ) );
    connect( action, SIGNAL(triggered()), &mainWindow, SLOT(editPS()) );

    // add actions to menu and store files
    FileRecord::List records;
    for( ActionMap::const_iterator iter = editionActions_.begin(); iter != editionActions_.end(); ++iter )
    {
        editionMenu_->addAction( iter->first );
        records << iter->second;
    }

    // additional actions
    editionMenu_->addSeparator();
    editionMenu_->addAction( tr( "Any" ), &mainWindow, SLOT(edit()) );

    // initalize thread and run.
    if( thread_.isRunning() ) return;
    thread_.setRecords( records );
    thread_.start();

}
